//---------------------------------------------------------------------------
// Copyright 2003 Andreas Huennebeck
//
// Licensed under the terms of the GNU General Public License.
// ABSOLUTELY NO WARRANTY. See the file 'COPYING' in this distribution.
//
// $Source: /home/ah/CvsTree/daten/src/corr/corr.c,v $
// $Id: corr.c,v 1.8 2004/04/21 20:38:33 ah Exp $ 
//
// Description: correlation of files
//
//---------------------------------------------------------------------------

static const char rcsid[] = "$Id: corr.c,v 1.8 2004/04/21 20:38:33 ah Exp $ (c) 2000 Andreas Huennebeck";

// --- necessary includes ---
#include <AH/Config/Platform.h> // always the first include
USING_NAMESPACE(AH)

#include <stdlib.h>
#include <math.h>
#include <iostream>

#include <AH/Config/Types.h>
#include <AH/Error/Class.h>
#include <AH/Error/Basic.h>
#include <AH/Error/Sys.h>
USING_NAMESPACE(AH)

#include "corr.h"

BEGIN_NAMESPACE(AH)

// --- local definitions ---

#define TL_CORR 0
#define AH_CORR 1
#define PP_CORR 0

#define TIMETRACE 1

// --- local constants ---

#ifdef REF_TYPE_DOUBLE
static const char REF_TYPE[] = "double";
#endif
#ifdef REF_TYPE_INT
static const char REF_TYPE[] = "int";
#endif
#ifdef REF_TYPE_BOOL
static const char REF_TYPE[] = "bool";
#endif

#ifdef IN_TYPE_DOUBLE
static const char IN_TYPE[] = "double";
#endif
#ifdef IN_TYPE_INT
static const char IN_TYPE[] = "int";
#endif
#ifdef IN_TYPE_BOOL
static const char IN_TYPE[] = "bool";
#endif

#ifdef OUT_TYPE_DOUBLE
static const char OUT_TYPE[] = "double";
#endif
#ifdef OUT_TYPE_INT
static const char OUT_TYPE[] = "int";
#endif


// --- local data ---

// --- local classes ---

// --- local functions ---


#if TL_CORR //************************************************************

void correlate (const corr_t* in, size_t in_size,
		const corr_t* ref, size_t ref_size,
		corr_t* out, size_t out_size,
		bool cyclic, int debon) 
{
    unsigned int h, i, j, k, modulo_limit, input_offset, circles, length,
start, end1, end2;

    /* Initialize out */
    memset (out, 0, out_size * sizeof (*out));

#if TIMETRACE
    const double msec_per_clock = 1000.0 / CLOCKS_PER_SEC;
    double start_time = (double)clock() * msec_per_clock;
#endif

    circles = in_size / out_size;
    length = ref_size;
    for (h = 0; h < circles; h++)
    {
        input_offset = h * length;
        start = input_offset;
        end2 = length + input_offset;
        for (i = 0, modulo_limit = length, end1 = length + input_offset;
             i < length; i++, modulo_limit--, end1--)
        {
            /*end = modulo_limit + input_offset;*/
            for (j = start, k = i; j < end1; ++j, ++k)
            {
                out[i] += in[j] * ref[k];
            }
            if (i > 0)
            {
                k = 0;
                for (; j < end2; ++j, ++k)
                {
                    out[i] += in [j] * ref[k];
                }
            }
        }
    }
#if TIMETRACE
    double corr_time = (double)clock() * msec_per_clock;
#endif
    // apply weighting factor:
    int wfact = circles * length;
    for (size_t h = 0; h < out_size; ++h)
	out[h] /= wfact;

#if TIMETRACE
    double weight_time = (double)clock() * msec_per_clock;
    std::cerr << "time for xcorr (wfact=" << wfact << "): "
	 << corr_time - start_time << " ms" << std::endl;
    std::cerr << "time for weighting (out_size=" << out_size << "): "
	 << weight_time - corr_time << " ms" << std::endl;
#endif
#if TIMETRACE1
    double corr_time = (double)clock() * msec_per_clock;
    std::cerr << "time for xcorr (circles=" << circles << "): "
	 << corr_time - start_time << " ms" << std::endl;
#endif
}
#endif // TL_CORR



#if AH_CORR //************************************************************

// made by AH and TL, improved by PP and AH
static void cyc_correlate
(
    const corr_ref_t* refbuf, const size_t refbuflen, 
    const corr_in_t* inbuf, const size_t inbuflen,
    corr_out_t* outbuf, const size_t outbuflen,
    int debon
) 
{
    if (debon) std::cerr << "doing circular correlation\n";

    const unsigned int MARK_POINT = 500;
 
#if TIMETRACE
    const double msec_per_clock = 1000.0 / CLOCKS_PER_SEC;
    double start_time = (double)clock() * msec_per_clock;
#endif

    // circular + averaging: number of loops is the maximum
    // of the three buffer sizes
    size_t kmax = refbuflen > inbuflen ? refbuflen : inbuflen;
    if (outbuflen > kmax) kmax = outbuflen;

    for (size_t k = 0; k < outbuflen; ++k)
	outbuf[k] = 0;
    for (size_t k = 0, out = 0; k < kmax; ++k, ++out)
    {
	// remember: modulos are slooow, especially for non-powers of 2
	if (out == outbuflen) out = 0;
	size_t coe = k;
	while (coe > inbuflen) coe -= inbuflen;
	for (size_t n = 0; n < refbuflen; ++n, ++coe)
	{
	    if (coe == inbuflen) coe = 0;
#ifdef REF_TYPE_BOOL
	    if (refbuf[n]) outbuf[out] += inbuf[coe];
	    else           outbuf[out] -= inbuf[coe];
#else
	    outbuf[out] += refbuf[n] * inbuf[coe];
#endif
	}
	
	if (debon && (0 == (k & MARK_POINT)))
	    std::cerr << k << "/ " << (k * 100) / kmax << "%\r";
    }

#if TIMETRACE
    double corr_time = (double)clock() * msec_per_clock;
#endif
#ifdef OUT_TYPE_DOUBLE
    // apply weighting factor:
    for (size_t k = 0; k < outbuflen; ++k)
	outbuf[k] /= kmax;
#endif

#if TIMETRACE
    double weight_time = (double)clock() * msec_per_clock;
    std::cerr << "time for xcorr-" << REF_TYPE << "-" << IN_TYPE << "-" << OUT_TYPE
	 << " (kmax=" << kmax << "): "
	 << corr_time - start_time << " ms" << std::endl;
    std::cerr << "time for weighting (outbuflen=" << outbuflen << "): "
	 << weight_time - corr_time << " ms" << std::endl;
#endif

    if (debon) std::cerr << outbuflen << "\n";
}


// made by AH and TL, improved by PP and AH
static void lin_correlate
(
    const corr_ref_t* refbuf, const size_t refbuflen, 
    const corr_in_t* inbuf, const size_t inbuflen,
    corr_out_t* outbuf, const size_t outbuflen,
    int debon
) 
{
    if (debon) std::cerr << "doing linear correlation\n";

    const unsigned int MARK_POINT = 500;
 
#if TIMETRACE
    const double msec_per_clock = 1000.0 / CLOCKS_PER_SEC;
    double start_time = clock() * msec_per_clock;
#endif

    // linear
    size_t kmax = refbuflen + inbuflen;
    for (size_t k = 0; k < kmax; ++k )
    {
	// remember: modulos are slooow, especially for non-powers of 2
	size_t coe = k;
	while (coe > inbuflen) coe -= inbuflen;
	for (size_t n = 0; n < refbuflen; ++n, ++coe)
	{
	    if (coe == inbuflen) coe = 0;
	    outbuf[k] += refbuf[n] * inbuf[coe];
	}
	// apply weighting factor:
	outbuf[k] /= refbuflen;
#if 0
	// according to Brook/Wynne: Signal processing 
	size_t coe = k;
	if (coe == inbuflen) break;
	size_t n;
	for (n = 0; n < refbuflen; ++n, ++coe)
	{
	    if (coe == inbuflen) break;
	    outbuf[k] += refbuf[n] * inbuf[coe];
	}
	// apply weighting factor:
	outbuf[k] /= n;
#endif
	if (debon && (0 == (k & MARK_POINT)))
	    std::cerr << k << "/ " << (k * 100) / kmax  << "%\r";
    }
    
#if TIMETRACE
    double corr_time = clock() * msec_per_clock;
    std::cerr << "time for linear xcorr (kmax=" << kmax << "): "
	 << corr_time - start_time << " ms" << std::endl;
#endif

    if (debon) std::cerr << outbuflen << "\n";
}



void correlate(const corr_ref_t* refbuf, const size_t refbuflen, 
	       const corr_in_t* inbuf, const size_t inbuflen,
	       corr_out_t* outbuf, const size_t outbuflen,
	       bool cyclic, int debon) 
{
    if (cyclic)
	cyc_correlate(refbuf, refbuflen,
		      inbuf, inbuflen,
		      outbuf, outbuflen,
		      debon); 
    else 
	lin_correlate(refbuf, refbuflen,
		      inbuf, inbuflen,
		      outbuf, outbuflen,
		      debon); 
}
#endif // AH_CORR



#if PP_CORR //************************************************************

void correlate(const corr_t* inbuf, const size_t inbuflen, 
	       const corr_t* coeffs, const size_t coeffslen,
	       corr_t* outbuf, const size_t outbuflen,
		bool cyclic, int debon) 
{
    /* clear output-buffer before processing */
    memset (outbuf, 0, outbuflen * sizeof (*outbuf));

    if ((inbuflen != outbuflen  &&  coeffslen != outbuflen)  &&
	(outbuflen < inbuflen + coeffslen - 1))
    {
	throw ErrorBasic("output buffer too small");
    }

    if (debon) 
	std::cerr << "doing " << (inbuflen == outbuflen ? "circular" : "linear")
	     << " correlation\n";

    const unsigned int MARK_POINT = 0xff;
 
    // PPs own code
    size_t  n;
    size_t  k;

    if (cyclic) {
        // circular (code not optimal)
        for ( n = 0; n < inbuflen; n++ ) {
            for ( k = 0; k < coeffslen; k++ )
                outbuf[(n + k) % inbuflen] += inbuf[n] * coeffs[coeffslen - k - 1];
        }
    } else {
        // linear
        for ( n = 0; n < inbuflen; n++ ) {
            for ( k = 0; k < coeffslen; k++ )
                outbuf[n + k] += inbuf[n] * coeffs[coeffslen - k - 1];
        }
    }
}
#endif // PP_CORR


END_NAMESPACE

// --- end of file ---
