//---------------------------------------------------------------------------
// Copyright 1998 Andreas Huennebeck
//
// Licensed under the terms of the GNU General Public License.
// ABSOLUTELY NO WARRANTY. See the file `COPYING' in this distribution.
//
// $Source: /home/ah/CvsTree/daten/src/include/AH/IO/Cdda/Sector.h,v $
// $Id: Sector.h,v 1.1 2005/01/11 00:28:09 ah Exp $
//
// Description: interface of class IOPlot
//
//---------------------------------------------------------------------------

#ifndef AH_IO_PLOT_H		// prevent multiple includes
#define AH_IO_PLOT_H

// --- forward declarations ---

// --- necessary includes ---

#include <AH/Config/Platform.h>
#include <AH/Config/Types.h>
USING_NAMESPACE(AH)

#include <string>

// --- types and classes ---

BEGIN_NAMESPACE(AH)


/** represents a CDDA sector value

   <tt>AH::CddaSector</tt> represents a CDDA sector value.
 */
class CddaSector
{
 public:
    static const uint32 BYTES_PER_SECOND = 44100 * 4;
    static const uint32 BYTES_PER_MINUTE = 44100 * 4 * 60;
    static const uint32 BYTES_PER_SECTOR = 2352;
    static const uint32 SECTORS_PER_SECOND = 75;

    /** default constructor. */
    CddaSector() : minutes(0), seconds(0), sectors(0) {}

    /** convert time value into CddaSector. */
    CddaSector(double time);

    /** convert byte value into CddaSector. */
    CddaSector(uint32 bytes);

    /** convert string into CddaSector. */
    CddaSector(const std::string& s);

    /** raw constructor. */
    CddaSector(uint32 minu, uint32 seco, uint32 sect)
	: minutes(minu), seconds(seco), sectors(sect)
	{ this->sanityCheck("CddaSector raw constructor"); }

    /** copy constructor. */
    CddaSector(const CddaSector& cs)
	: minutes(cs.minutes), seconds(cs.seconds), sectors(cs.sectors) {}

    /** return value as string. */
    std::string asString() const;

    /** return value as bytes. */
    uint32 asBytes() const;

    /** return value as time (seconds). */
    double asTime() const;

    /** overloaded operator = */
    CddaSector& operator = (const CddaSector &cs);

    /** overloaded operator += for addition of two CDDA sector values. */
    CddaSector& operator += (const CddaSector &cs);

    /** overloaded operator -= for addition of two CDDA sector values. */
    CddaSector& operator -= (const CddaSector &cs);

    /** overloaded operator * for multiplication of CddaSector with double. */
    CddaSector& operator *= (double k);

    /** overloaded operator / for division of CddaSector by double. */
    CddaSector& operator /= (double k);

    /** overloaded operator + for addition of two CDDA sector values. */
    friend CddaSector operator + (const CddaSector &cs1, 
				  const CddaSector &cs2);

    /** overloaded operator - for subtraction of two CDDA sector values. */
    friend CddaSector operator - (const CddaSector &cs1, 
				  const CddaSector &cs2);

    /** overloaded operator * for multiplication of CddaSector with double. */
    friend CddaSector operator * (const CddaSector &cs, double k);

    /** overloaded operator * for multiplication of double with CddaSector. */
    friend CddaSector operator * (double k, const CddaSector &cs);

    /** overloaded operator / for division of CddaSector by double. */
    friend CddaSector operator / (const CddaSector &cs, double k);

    /** overloaded operator == for comparison of two CddaSectors. */
    friend bool operator == (const CddaSector &cs1, const CddaSector& cs2);

    /** overloaded operator != for comparison of two CddaSectors. */
    friend bool operator != (const CddaSector &cs1, const CddaSector& cs2);

    /** overloaded operator < for comparison of two CddaSectors. */
    friend bool operator < (const CddaSector &cs1, const CddaSector& cs2);

    /** overloaded operator > for comparison of two CddaSectors. */
    friend bool operator > (const CddaSector &cs1, const CddaSector& cs2);

 private:
    uint32 minutes;
    uint32 seconds;
    uint32 sectors;

    void sanityCheck(const std::string& msg) const;
};



END_NAMESPACE

#endif // !AH_IO_PLOT_H



// --- function prototypes ---


// --- end of file ---
