//---------------------------------------------------------------------------
// Copyright 1998 Andreas Huennebeck
//
// Licensed under the terms of the GNU General Public License.
// ABSOLUTELY NO WARRANTY. See the file `COPYING' in this distribution.
//
// $Source: /home/ah/CvsTree/daten/src/include/AH/IO/CmdOption.h,v $
// $Id: CmdOption.h,v 1.2 2004/05/04 23:20:23 ah Exp $
//
// Description: interface of class IOCmdOption
//
//---------------------------------------------------------------------------

#ifndef AH_IO_CMD_OPTION_H		// prevent multiple includes
#define AH_IO_CMD_OPTION_H

// --- forward declarations ---

// --- necessary includes ---

#include <AH/Config/Platform.h>
USING_NAMESPACE(AH)

#include <string>
#include <vector>
USING_NAMESPACE(std)

// --- types and classes ---

BEGIN_NAMESPACE(AH)

/**
   <tt>AH::IOCmdOption</tt> ...

   Please include <em>AH/IO/CmdOption.h</em>.
 */
class IOCmdOption
{
 public:
    enum SecondArg 
    {
	/** no second argument. */
	SecondArgNone, 
	/** needs a second argument. */
	SecondArgAlways,
	/** may have a second argument. */
	SecondArgOptional
    };

    // in client: override if necessary in subclasses ----------

    /** constructor. */
    IOCmdOption(char short_opt, const char* long_opt, SecondArg sec,
		const std::string& desc)
	: shortOpt(short_opt), longOpt(long_opt), secArg(sec),
	  description(desc), first(0) {}

    /** destructor */
    virtual ~IOCmdOption() {}

    // in client: override if necessary, do not use -------------

    /** set option to set. */
    virtual void set()
	{ ++first; }

    /** set second argument. */
    virtual void setSecArg(const std::string& arg)
	{ second.push_back(arg); }

    // used by client only: -------------------------------------

    /** return true if option is set. */
    int isSet() const 
	{ return first; }

    /** return second argument. */
    std::vector<std::string> getSecArg() const
	{ return second; }

    /** return last second argument. */
    std::string getLastSecArg() const
	{ return (second.size() ? second[second.size()-1] : ""); }

    // used by IOCmdOptHandler only: ----------------------------

    /** return true if it is a short option. */
    bool isShort() const
	{ return (shortOpt != '\0'); }

    /** return true if it is a long option. */
    bool isLong() const
	{ return (longOpt != 0); }

    /** return second argument. */
    SecondArg getSecArgMode() const
	{ return secArg; }

    /** return short option char. */
    char getShortOpt() const
	{ return shortOpt; }

    /** return long option name. */
    std::string getLongOpt() const
	{ return longOpt; }
   
    /** return description. */
    std::string getDescription() const
	{ return description; }
   
 protected:
    char        shortOpt;
    const char* longOpt;
    SecondArg   secArg;
    std::string description;

 private:
    int first;
    std::vector<std::string> second;
};	// IOCmdOption



END_NAMESPACE

#endif // !AH_IO_CMD_OPTION_H

















