//---------------------------------------------------------------------------
// Copyright 2004 Andreas Huennebeck
//
// Licensed under the terms of the GNU General Public License.
// ABSOLUTELY NO WARRANTY. See the file 'COPYING' in this distribution.
//
// $Source: /home/ah/CvsTree/daten/src/include/AH/IO/Midi/RawCmd.h,v $
// $Id: RawCmd.h,v 1.2 2004/04/20 22:25:30 ah Exp $
//
// Description: class representing a raw MIDI command. 
//
//---------------------------------------------------------------------------

#ifndef AH_IO_MIDI_RAW_CMD_H		// prevent multiple includes
#define AH_IO_MIDI_RAW_CMD_H

// --- forward declarations ---

// --- necessary includes ---

#include <AH/Config/Platform.h>
#include <AH/Config/Types.h>
USING_NAMESPACE(AH)

#include <cstdio>
#include <vector>
#include <string>

// --- types and classes ---

BEGIN_NAMESPACE(AH)

// represents a raw MIDI command
class IOMidiRawCmd
{
 public:
    //-------------------
    // con-/destructors:
    //-------------------
    
    /// constructor for raw MIDI command 
    IOMidiRawCmd
    (
	const std::vector<uint8>& sendData,
	int32 recSize = -1
    )
	: txData(sendData), rxSize(recSize) {}

    /// constructor for raw MIDI command 
    IOMidiRawCmd
    (
	const uint* sendData,
	int32 sendSize,
	int32 recSize = -1
    );

    /// constructor for empty raw MIDI command 
    IOMidiRawCmd
    (
	int32 recSize = -1
    )
	: rxSize(recSize) {}

    /// destructor
    virtual ~IOMidiRawCmd() {}

    //------------------
    // access functions
    //------------------
    
    /// get send data
    const std::vector<uint8>& getSendData() const { return txData; }
    /// get size of send data
    uint32 getSendDataSize() const { return txData.size(); }
    /// get received data
    const std::vector<uint8>& getRecData() const { return rxData; }
    /// get size of received data
    uint32 getRecDataSize() const { return rxData.size(); }
    /// get desired size of receive data
    int32 getWishRecDataSize() const { return rxSize; }

    /// get send data as hex string (for debug)
    std::string getSendDataHex() const { return getDataHex(txData); }
    /// get received data as hex string (for debug)
    std::string getRecDataHex() const { return getDataHex(rxData); }

    //---------------
    // set functions
    //---------------
    virtual void pushSendByte(uint8 byte) { txData.push_back(byte); }
    virtual void pushRecByte(uint8 byte) { rxData.push_back(byte); }

 protected:
    std::vector<uint8> txData;
    std::vector<uint8> rxData;
    int32 rxSize;

 private:
    /// get data as hex string (for debug)
    std::string getDataHex(const std::vector<uint8>& data) const;
};

END_NAMESPACE

#endif // !AH_IO_MIDI_RAW_CMD_H
