//---------------------------------------------------------------------------
// Copyright 1997 Andreas Huennebeck
//
// $Source: /home/ah/CvsTree/daten/src/libAHetech/NoneInvAmp.c,v $
// $Id: NoneInvAmp.c,v 1.6 2001/10/05 23:44:26 ah Exp $
//
// Description: implementation of class NoneInvAmp
//
//---------------------------------------------------------------------------

static const char rcsid[] = "$Id: NoneInvAmp.c,v 1.6 2001/10/05 23:44:26 ah Exp $ (c) 1997 Andreas Huennebeck";

// --- necessary includes ---

#include <AH/Config/Platform.h>
#include <AH/Etech/NoneInvAmp.h>

// --- local stuff ---

// --- class member implementation ---

// get open loop amplification at frequency {\tt freq}
const Cplx*  NoneInvAmp::getAd(Dbl freq)
{
    return op.getAd(freq);
}

// get amplification at frequency 'freq':
Cplx NoneInvAmp::getA(Dbl freq)
{
    const Cplx* Ad = getAd(freq);
    if (!Ad) return ((Z1 + Z2)/Z2);

    bool hasZin = true;
    Impedance in;

    const Impedance* opin = op.getZin(freq);
    if (opin)
    {
	if (Zin) in = *opin || *Zin;
	else	 in = *opin;
    }
    else
    {
	if (Zin) in = *Zin;
	else	 hasZin = false;
    }

    Impedance term2;
    const Impedance* Zo = op.getZout(freq);
    if (Zo) 
    {
	Cplx term1(0);
	if (hasZin)
	    term1 = ((Z1 + *Zo) * Z2) / in;
	term2 = Z1 + Z2 + *Zo + Impedance(term1);
    }
    else
    {
	Cplx term1(0);
	if (hasZin)
	    term1 = (Z1 * Z2) / in;
	term2 = Z1 + Z2 + Impedance(term1);
    }
    Cplx Acl = (Z1 + Z2) / (Z2 + term2 / *Ad);
    return (Acl);
}

// get amplification at frequency 'freq' with Load:
Cplx NoneInvAmp::getA(Dbl freq, const Impedance& load)
{
    const Cplx* Ad = op.getAd(freq);
    if (!Ad) return ((Z1 + Z2)/Z2);

    bool hasZin = true;
    Impedance in;

    const Impedance* opin = op.getZin(freq);
    if (opin)
    {
	if (Zin) in = *opin || *Zin;
	else	 in = *opin;
    }
    else
    {
	if (Zin) in = *Zin;
	else	 hasZin = false;
    }

    Impedance Zs(0);
    Impedance nenner;

    const Impedance* Zo = op.getZout(freq);
    if (Zo) 
    {
	Cplx term1(0);
	Cplx term2(1);
	if (hasZin)
	{
	    Cplx term3 = *Zo/load * ((Z2 + Zs) * Z1 + Z2 * Zs * Zs / Z1);
	    term1 = ((Z1 + *Zo) * Z2 + term3) / in;
	    term2 = 1 + Zs / in;
	}

	Impedance term4 = (Z1 + Z2) * (*Zo/load);
	Impedance term5 = (Z1 + Z2 + *Zo) * term2;
	nenner = Z2 + (term5 + term4 + Impedance(term1)) / *Ad;
    }
    else
    {
	Cplx term1(0);
	Cplx term2(1);
	if (hasZin)
	{
	    term1 = (Z1 * Z2) / in;
	    term2 = 1 + Zs / in;
        }

        Impedance term5 = (Z1 + Z2 + *Zo) * term2;
	nenner = Z2 + (term5 + Impedance(term1)) / *Ad;
    }
    Cplx Acl = (Z1 + Z2) / nenner;
    return (Acl);
}

// --- friend functions ---

// --- overloaded operators ---

