//---------------------------------------------------------------------------
// Copyright 1999 Andreas Huennebeck
//
// Licensed under the terms of the GNU General Public License.
// ABSOLUTELY NO WARRANTY. See the file `COPYING' in this distribution.
//
// $Source: /home/ah/CvsTree/daten/src/libAHio/RiffWaveDataBuf.c,v $
// $Id: 
//
// Description: 
//
//---------------------------------------------------------------------------

static const char cvsid[] = "$Id: RiffWaveDataBuf.c,v 1.1 2000/02/27 23:49:01 ah Exp $ (c) 1999 Andreas Huennebeck";

// --- necessary includes ---

#include <AH/IO/Riff/WaveDataBuf.h>
#include <AH/Error/Intern.h>
#include <AH/Error/Sys.h>
USING_NAMESPACE(AH)

BEGIN_NAMESPACE(AH)

// constructor for buffer of {\tt n} elements of size {\tt bits} bits
IORiffWaveDataBuffer::IORiffWaveDataBuffer(size_t n, size_t bits)
    : bytes(bits/8), curnum(n), maxnum(curnum), 
      shift(32 - bits), mask((1 << bits) - 1)
{
    if ((bits % 8) || (bytes > 4)) throw ErrorInternBadArg();

    cursiz = maxsiz = maxnum * bytes;
    buf = new uint8[maxsiz];
    checkBuf();
}

// resize buffer for {\tt n} elements
void IORiffWaveDataBuffer::resize(size_t n)
{
    if (n == curnum) return;
    if (n > maxnum) 
	recreate(n);
    else
    {
	curnum = n;
	cursiz = curnum * bytes;
    }
}

// resize buffer for {\tt n} elements
void IORiffWaveDataBuffer::recreate(size_t n)
{
    if (n == curnum) return;
    if (buf) delete [] buf;
    maxnum = curnum = n;
    cursiz = maxsiz = maxnum * bytes;
    buf = new uint8[maxsiz];
    checkBuf();
}

// return specified entry in buffer
int32 IORiffWaveDataBuffer::getEntry(size_t idx) const 
{
    if (idx < curnum)
    {
	int32 val = *(int32*)(buf + (idx * bytes));
	if (shift)
	{
	    val >>= shift;
	    val &= mask;
	}
	return val;
    }
    else
	throw ErrorInternBadArg();
}

// check if buffer could be created
void IORiffWaveDataBuffer::checkBuf()
{
    if (!buf) 
    {
	maxnum = curnum = maxsiz = cursiz = 0;
	throw ErrorSys(ENOMEM);
    }
}


END_NAMESPACE
