//---------------------------------------------------------------------------
// Copyright 1999 Andreas Huennebeck
//
// Licensed under the terms of the GNU General Public License.
// ABSOLUTELY NO WARRANTY. See the file `COPYING' in this distribution.
//
// $Source: /home/ah/CvsTree/daten/src/libAHio/RiffWaveFmtChunk.c,v $
// $Id: 
//
// Description: 
//
//---------------------------------------------------------------------------

static const char rcsid[] = "$Id: RiffWaveFmtChunk.c,v 1.3 2004/04/20 22:26:22 ah Exp $ (c) 1999 Andreas Huennebeck";

// --- necessary includes ---

#include <AH/Config/Platform.h>
#include <AH/IO/Riff/WaveFmtChunk.h>
#include <AH/IO/Convert.h>
#include <AH/Error/Sys.h>
USING_NAMESPACE(AH)

#include <string>
#include <cerrno>
#include <iostream>
USING_NAMESPACE(std)

BEGIN_NAMESPACE(AH)

ostream& IORiffWaveFmtChunk::Specific::printFormat(ostream& out, const char* pre)
{
    if (pre) out << pre;
    out << "format tag  = " << key << endl;
    return out;
}

ostream& IORiffWaveFmtChunk::MsPcm::printFormat(ostream& out, const char* pre)
{
    if (pre) out << pre;
    out << "format      = Microsoft PCM" << endl;
    return out;
}

ostream& IORiffWaveFmtChunk::MsPcm::printInfo(ostream& out, const char* pre)
{
    if (pre) out << pre;
    out << "sample size = " << *sampleSize << " Bits" << endl;
    return out;
}

IORiffWaveFmtChunk::IORiffWaveFmtChunk(const unsigned char* data)
    : IORiffChunk(data), 
      format(
	  IOConvert::LittleEndCharsToUint16(IORiffChunk::chunkData())),
      channels(
	  IOConvert::LittleEndCharsToUint16(IORiffChunk::chunkData()+2)),
      sampleRate(
	  IOConvert::LittleEndCharsToUint32(IORiffChunk::chunkData()+4)),
      dataRate(
	  IOConvert::LittleEndCharsToUint32(IORiffChunk::chunkData()+8)),
      blockSize(
	  IOConvert::LittleEndCharsToUint16(IORiffChunk::chunkData()+12))
{
    switch (format)
    {
    default:     specData = new Specific(format);
    case 0x0001: specData = new MsPcm(chunkData());
    }
}

ostream& IORiffWaveFmtChunk::printInfo(ostream& out, const char* pre)
{
    IORiffChunk::printInfo(out, pre);
    if (specData) 
    {
	specData->printFormat(out, pre);
    }
    else
    {
	if (pre) out << pre;
	out << "format tag  = " << format << endl;
    }
    if (pre) out << pre;
    out << "channels    = " << channels << endl;
    if (pre) out << pre;
    out << "sample rate = " << sampleRate << "/sec" << endl;
    if (pre) out << pre;
    out << "data rate   = " << dataRate << " bytes/sec" << endl;
    if (pre) out << pre;
    out << "block size  = " << blockSize << endl;
    if (specData) specData->printInfo(out, pre);

    return out;
}

END_NAMESPACE

