//---------------------------------------------------------------------------
// Copyright 1999 Andreas Huennebeck
//
// Licensed under the terms of the GNU General Public License.
// ABSOLUTELY NO WARRANTY. See the file `COPYING' in this distribution.
//
// $Source: /home/ah/CvsTree/daten/src/libAHio/TestRiffWaveFile.c,v $
// $Id: 
//
// Description: 
//
//---------------------------------------------------------------------------

static const char rcsid[] = "$Id: TestRiffWaveFile.c,v 1.2 2004/04/20 22:26:22 ah Exp $ (c) 1999 Andreas Huennebeck";

// --- necessary includes ---

#include <AH/IO/Riff/WaveFile.h>	// my interface first, says Lakos
#include <AH/Error/Class.h>
#include <AH/IO/Plot.h>
#include <AH/IO/CFormat.h>
USING_NAMESPACE(AH)

#include <cstdlib>
#include <unistd.h>
#include <string>
#include <cerrno>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h> 
#include <iostream>
#include <fstream>
USING_NAMESPACE(std)

typedef IOCFormat Cfm;


static void askUser (const string& question, string& answer)
{
    //-----------------------
    // ask user for command:
    //-----------------------
    cout << question;
    cout.flush();
    if (!getline(cin, answer))
    {
	cerr << "- Terminal I/O Error" << endl;
	return;
    }
}



static unsigned int execute_test
(
    const char*	wavename,
    const char*	dataname,
    size_t bufsize,
    ostream& out,
    double t1,
    double t2
)
{
    try
    {
	IORiffWaveFile wave(wavename);
	wave.printInfo(cerr);

	string notused;
	IOPlot plot;

	wave.printData(out,1024,t1,t2);
	plot.exec("set grid");
	plot.setYrange(-32768, 32768);
	plot.setXrange(t1, t2);

	plot.plot(dataname, "using 2:3 title \"Left\" with lines");
	askUser("Inhalt der Wav-Datei (Left) <ret>", notused);
	plot.plot(dataname, "using 2:4 title \"Right\" with lines");
	askUser("Inhalt der Wav-Datei (Right) <ret>", notused);
	plot.plot(dataname, "using 2:5 title \"Left - Right\" with lines");
	askUser("Inhalt der Wav-Datei (Left - Right) <ret>", notused);

	int16	lMin;
	int16	lMax;
	int16	rMin;
	int16	rMax;
	int32	lMinCnt;
	int32	lMaxCnt;
	int32	rMinCnt;
	int32	rMaxCnt;
	wave.getLimits(t1, t2, lMin, lMax, rMin, rMax, 
		       lMinCnt, lMaxCnt, rMinCnt, rMaxCnt);
	Cfm val("%6d");
	Cfm cnt("%10d");
	cerr << "Range and Limits:\n" 
	     << " L: range = [" << val << lMin << ".." << val << lMax << "] "
	     << " counts =  [" << cnt << lMinCnt << "/" << cnt << lMaxCnt 
	     << "]\n"
	     << " R: range = [" << val << rMin << ".." << val << rMax << "] "
	     << " counts =  [" << cnt << rMinCnt << "/" << cnt << rMaxCnt 
	     << "]"
	     << endl;
    }
    catch(ErrorClass& error)
    {
	cerr << "Error processing " << wavename << ": " << error.getMsg() 
	     << endl;
abort();
	return 1;
    }
    return 0;
}



int main(int argc, char* argv[])
{
    cerr << "Test of " << argv[0] << ":" << endl;

    // set default options:
    static char* cmdLineOptions = "b:o:f:t:";
    ostream* out = &cout;

    // loop through command line options:
    int option;
    size_t bufsize = 1024;
    double t1 = 20;
    double t2 = 30;
    ofstream fout;
    const char* outname = "data";
    while ((option = getopt(argc, argv, cmdLineOptions)) != -1)
    {
	switch (option) 
	{
	case 'b': bufsize = atoi(optarg);
	          if (0 == bufsize) bufsize = 1024;
		  break;

	case 'f': t1 = atof(optarg);			break;
	case 't': t2 = atof(optarg);			break;

	case 'o': fout.open(optarg);
	          if (!fout.is_open()) 
		  {
		      cerr << "Cannot open " << optarg << ": " 
			  << strerror(errno) << endl;
		      return 1;
		  }
		  outname = optarg;
		  out = &fout;
		  break;

	case '?': /* fall through */
	default : exit(1);				break;
	}
    }

    unsigned int errnum = 0;
    for (int i=optind; i < argc; i++)
    {
	const char* filename = argv[i];
	cerr << "processing " << filename 
	     << " [" << t1 << " .. " << t2 << "] ..." << endl;
	errnum += execute_test(filename, outname, bufsize, *out, t1, t2);
    }

    // print test results:
    cerr << argv[0] << ": ";
    if (errnum)
    {
	cerr << "failed with " << errnum << " error"
	     << (errnum > 1 ? "s" : "") << ". stop." << endl;
	return 1;;
    }
    cerr << "passed." << endl;
    return 0;
}
